#!/bin/sh
# ServiceScan + TSDuck Monitor Script
# Monitors for ServiceScan reports and triggers TSDuck analysis

# Define scan directories to monitor
XML_DIR="/media/usb/ServiceScan"
[ ! -d "$XML_DIR" ] && XML_DIR="/hdd/ServiceScan"
[ ! -d "$XML_DIR" ] && XML_DIR="/media/FTA/ServiceScan"
[ ! -d "$XML_DIR" ] && XML_DIR="/tmp"

# Read settings from Enigma2 config
SETTINGS_FILE="/etc/enigma2/settings"
if [ -f "$SETTINGS_FILE" ]; then
    # Extract scan mode setting
    SCAN_MODE=$(grep "config.plugins.scantsduck.scan_mode" "$SETTINGS_FILE" | cut -d= -f2)
    # Default to passive if not found
    [ -z "$SCAN_MODE" ] && SCAN_MODE="passive"
else
    # Default to passive mode if settings file not found
    SCAN_MODE="passive"
fi

# Create log directory
LOG_DIR="/tmp/scantsduck"
mkdir -p $LOG_DIR

# Log file
LOG_FILE="$LOG_DIR/scan_monitor.log"

# Log function
log() {
    echo "$(date '+%Y-%m-%d %H:%M:%S') - $1" >> "$LOG_FILE"
    echo "$(date '+%Y-%m-%d %H:%M:%S') - $1"
}

# Force file system sync to make files visible
force_sync() {
    # Sync to ensure files are written to disk
    sync
    
    # If you have root permissions, try to flush caches
    if [ "$(id -u)" = "0" ]; then
        echo 3 > /proc/sys/vm/drop_caches 2>/dev/null || true
    fi
    
    # Update directory timestamp to force reread
    touch "$XML_DIR"
    
    # Force filesystem refresh by listing directory contents
    ls -la "$XML_DIR" > /dev/null 2>&1
    
    # Force remount of filesystem if possible (needs root)
    if [ "$(id -u)" = "0" ]; then
        DIR_MOUNT=$(df "$XML_DIR" | tail -1 | awk '{print $6}')
        if [ -n "$DIR_MOUNT" ] && [ "$DIR_MOUNT" != "/" ]; then
            mount -o remount "$DIR_MOUNT" 2>/dev/null || true
            log "Remounted filesystem at $DIR_MOUNT"
        fi
    fi
    
    log "Forced file system sync"
}

log "Starting ServiceScan TSDuck monitor (improved version)"
log "Monitoring directory: $XML_DIR"
log "Scan mode: $SCAN_MODE"
log "Process running as: $(id)"

# Make sure monitor directory exists and has correct permissions
if [ ! -d "$XML_DIR" ]; then
    mkdir -p "$XML_DIR"
    chmod 755 "$XML_DIR"
    log "Created monitoring directory: $XML_DIR"
else
    # Ensure directory is accessible
    chmod 755 "$XML_DIR" 2>/dev/null || log "Warning: Could not set permissions on $XML_DIR"
    log "Using existing directory: $XML_DIR"
fi

# Check for in-progress scans (active mode)
is_scan_in_progress() {
    pidof enigma2 >/dev/null && (pidof enigma2scanner >/dev/null || pgrep -f "servicescanning" >/dev/null)
}

# Simple monitoring loop (in production, would use inotify)
while true; do
    # Create log directory if it doesn't exist (needed after reboot)
    mkdir -p "$LOG_DIR"
    
    # Check active mode - monitor scan process
    if [ "$SCAN_MODE" = "active" ]; then
        # In active mode, check if scan is in progress
        if is_scan_in_progress; then
            log "Scan in progress detected (active mode)..."
            # Wait for scan to complete
            sleep 5
            # Continue monitoring
            continue
        fi
    fi
    
    # Force sync to make any new files visible
    force_sync
    
    # Check for new scan files using multiple methods
    log "Checking for new scan files in $XML_DIR..."
    
    # Method 1: Use find with wider pattern match
    for FILE in $(find "$XML_DIR" -maxdepth 1 -name "*.xml" -mmin -60 2>/dev/null); do
        # Only process files we haven't seen before
        PROCESSED_MARK="${FILE}.processed"
        if [ ! -f "$PROCESSED_MARK" ]; then
            # Check if it looks like a ServiceScan file
            if grep -q "<Satellite\s*=" "$FILE" || grep -q "Tp#" "$FILE"; then
                log "ServiceScan report detected: $FILE"
                
                # Run analyzer
                log "Running TSDuck analysis..."
                python3 /usr/bin/scan_analyzer.py "$FILE" >> "$LOG_FILE" 2>&1
                RESULT=$?
                
                if [ $RESULT -eq 0 ]; then
                    log "Analysis completed successfully"
                else
                    log "Error: Analysis failed with exit code $RESULT"
                fi
                
                # Touch processed mark
                touch "$PROCESSED_MARK"
                chmod 644 "$PROCESSED_MARK" 2>/dev/null
                log "Marked file as processed"
                
                # Force newer timestamp on enhanced HTML and TXT to make them appear at top of list
                ENHANCED_HTML="${FILE}.enhanced.html"
                ENHANCED_TXT="${FILE}.enhanced.txt"
                
                # Wait for files to be created and visible (up to 5 seconds)
                MAX_WAIT=5
                WAIT_COUNT=0
                while [ $WAIT_COUNT -lt $MAX_WAIT ]; do
                    if [ -f "$ENHANCED_HTML" ] && [ -f "$ENHANCED_TXT" ]; then
                        break
                    fi
                    log "Waiting for enhanced report files to be created ($WAIT_COUNT/$MAX_WAIT)..."
                    sleep 1
                    WAIT_COUNT=$((WAIT_COUNT + 1))
                done
                
                # Update file timestamps
                if [ -f "$ENHANCED_HTML" ]; then
                    touch "$ENHANCED_HTML"
                    chmod 644 "$ENHANCED_HTML" 2>/dev/null
                    log "Found HTML report: $ENHANCED_HTML"
                else
                    log "Warning: HTML report not created: $ENHANCED_HTML"
                fi
                
                if [ -f "$ENHANCED_TXT" ]; then
                    touch "$ENHANCED_TXT"
                    chmod 644 "$ENHANCED_TXT" 2>/dev/null
                    log "Found TXT report: $ENHANCED_TXT"
                else
                    log "Warning: TXT report not created: $ENHANCED_TXT"
                fi
                
                # Force sync after processing a file
                force_sync
                
                # Final check if files are visible
                if [ -f "$ENHANCED_HTML" ] && [ -f "$ENHANCED_TXT" ]; then
                    # Create symbolic links in /tmp for easy access
                    TMP_LINK_HTML="/tmp/$(basename "$ENHANCED_HTML")"
                    TMP_LINK_TXT="/tmp/$(basename "$ENHANCED_TXT")"
                    
                    # Create links only if target files exist
                    ln -sf "$ENHANCED_HTML" "$TMP_LINK_HTML" 2>/dev/null
                    ln -sf "$ENHANCED_TXT" "$TMP_LINK_TXT" 2>/dev/null
                    
                    log "Enhanced reports are visible and linked in /tmp"
                else
                    log "Warning: Enhanced reports still not visible after multiple attempts"
                    
                    # Final desperate attempt - copy to /tmp which is always accessible
                    TMP_HTML="/tmp/$(basename "$ENHANCED_HTML")"
                    TMP_TXT="/tmp/$(basename "$ENHANCED_TXT")"
                    
                    # Check if original file exists but is not visible
                    if [ -f "$FILE" ]; then
                        log "Running emergency analyzer directly to /tmp..."
                        python3 /usr/bin/scan_analyzer.py "$FILE" "/tmp" >> "$LOG_FILE" 2>&1
                    fi
                fi
                
                # Final sync to ensure everything is flushed
                force_sync
            fi
        fi
    done
    
    # Clean up old logs
    find "$LOG_DIR" -name "*.log" -mtime +7 -delete 2>/dev/null
    
    # Sleep for 30 seconds
    sleep 30
    
    # Periodically force sync to ensure visibility
    force_sync
done