# -*- coding: utf-8 -*-
from Screens.Screen import Screen
from Components.ConfigList import ConfigListScreen
from Components.config import config, ConfigSubsection, ConfigYesNo, getConfigListEntry, ConfigSelection
from Components.ActionMap import ActionMap
from Components.Label import Label
from Components.Sources.StaticText import StaticText
from Plugins.Plugin import PluginDescriptor
from Screens.MessageBox import MessageBox
from enigma import eTimer
import os
import subprocess
import time

# Configuration
config.plugins.scantsduck = ConfigSubsection()
config.plugins.scantsduck.autostart = ConfigYesNo(default=False)
config.plugins.scantsduck.show_in_extensionsmenu = ConfigYesNo(default=True)
config.plugins.scantsduck.scan_mode = ConfigSelection(
    choices=[
        ("passive", _("Passive - Only analyze after scan")),
        ("active", _("Active - Analyze during scan"))
    ], 
    default="passive"
)

# Main settings screen
class ScanTSDuckSetup(ConfigListScreen, Screen):
    skin = """
    <screen position="center,center" size="952,680" title="ServiceScan TSDuck Integration" >
        <ePixmap pixmap="skin_default/buttons/red.png" position="0,0" size="238,68" alphatest="on" />
        <ePixmap pixmap="skin_default/buttons/green.png" position="238,0" size="238,68" alphatest="on" />
        <ePixmap pixmap="skin_default/buttons/yellow.png" position="476,0" size="238,68" alphatest="on" />
        <widget source="key_red" render="Label" position="0,0" zPosition="1" size="238,68" font="Regular;34" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
        <widget source="key_green" render="Label" position="238,0" zPosition="1" size="238,68" font="Regular;34" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
        <widget source="key_yellow" render="Label" position="476,0" zPosition="1" size="238,68" font="Regular;34" halign="center" valign="center" backgroundColor="#a08500" transparent="1" />
        <widget name="config" position="8,85" size="935,340" itemHeight="38" font="Regular;30" scrollbarMode="showOnDemand" />
        <eLabel position="8,442" size="935,2" backgroundColor="#aaaaaa" />
        <widget name="status" position="8,459" size="935,119" font="Regular;27" />
        <widget name="reports" position="8,595" size="935,68" font="Regular;27" />
    </screen>"""
    
    def __init__(self, session):
        Screen.__init__(self, session)
        self.session = session
        self.setTitle(_("ServiceScan TSDuck Integration"))
        
        # Initialize buttons
        self["key_red"] = StaticText(_("Cancel"))
        self["key_green"] = StaticText(_("Save"))
        self["key_yellow"] = StaticText(_("View Reports"))
        
        # Initialize status label
        self["status"] = Label()
        self["reports"] = Label()
        self.updateStatus()
        
        # Initialize ConfigListScreen
        ConfigListScreen.__init__(self, [])
        self.createSetup()
        
        # Initialize actions
        self["actions"] = ActionMap(["SetupActions", "ColorActions"],
        {
            "green": self.save,
            "red": self.cancel,
            "yellow": self.viewReports,
            "cancel": self.cancel,
            "ok": self.save,
        }, -2)
        
        # Update timer
        self.timer = eTimer()
        self.timer.callback.append(self.updateStatus)
        self.timer.start(5000)
    
    def createSetup(self):
        self.list = []
        self.list.append(getConfigListEntry(_("Start scan analyzer automatically"), config.plugins.scantsduck.autostart))
        self.list.append(getConfigListEntry(_("Show in extensions menu"), config.plugins.scantsduck.show_in_extensionsmenu))
        self.list.append(getConfigListEntry(_("Analysis mode"), config.plugins.scantsduck.scan_mode))
        
        self["config"].list = self.list
        self["config"].l.setList(self.list)
    
    def updateStatus(self):
        status = ""
        
        # Check if TSDuck is installed
        if self.isTSDuckInstalled():
            status += _("TSDuck: Installed\n")
        else:
            status += _("TSDuck: Not installed - please install tsduck-minimal\n")
        
        # Check if ScanAnalyzer service is running
        if self.isAnalyzerRunning():
            status += _("Analyzer service: Running\n")
        else:
            status += _("Analyzer service: Stopped\n")
        
        self["status"].setText(status)
        
        # Check if there are recent scan reports
        recent_reports = self.getRecentReports()
        if recent_reports:
            self["reports"].setText(_("Recent scans: %d (newest: %s)") % (len(recent_reports), recent_reports[0]))
        else:
            self["reports"].setText(_("No recent scan reports found"))
    
    def isTSDuckInstalled(self):
        try:
            result = subprocess.run(['which', 'tsanalyze'], 
                                   capture_output=True, text=True)
            return result.returncode == 0
        except:
            return False
    
    def isAnalyzerRunning(self):
        try:
            result = subprocess.run(['pgrep', '-f', 'scan_monitor.sh'], 
                                   capture_output=True, text=True)
            return result.returncode == 0
        except:
            return False
    
    def getRecentReports(self):
        scan_dirs = [
            "/media/usb/ServiceScan",
            "/hdd/ServiceScan",
            "/media/FTA/ServiceScan",
            "/tmp"
        ]
        
        reports = []
        for directory in scan_dirs:
            if os.path.exists(directory):
                for file in sorted(os.listdir(directory)):
                    if file.endswith(".enhanced.html"):
                        reports.append(file)
        
        return reports[:5]  # Return 5 most recent
    
    def startAnalyzerService(self):
        # Check if the service is already running
        if not self.isAnalyzerRunning():
            subprocess.Popen(['/usr/bin/scan_monitor.sh'], 
                            stdout=open('/tmp/scantsduck/stdout.log', 'w'),
                            stderr=open('/tmp/scantsduck/stderr.log', 'w'))
    
    def stopAnalyzerService(self):
        if self.isAnalyzerRunning():
            subprocess.run(['killall', 'scan_monitor.sh'])
    
    def save(self):
        for x in self["config"].list:
            x[1].save()
        
        # Handle service startup/shutdown
        if config.plugins.scantsduck.autostart.value:
            self.startAnalyzerService()
        else:
            self.stopAnalyzerService()
        
        self.close()
    
    def cancel(self):
        for x in self["config"].list:
            x[1].cancel()
        self.close()
    
    def viewReports(self):
        # Get available reports
        reports = self.getRecentReports()
        
        if not reports:
            self.session.open(MessageBox, _("No enhanced reports found."), MessageBox.TYPE_INFO, timeout=5)
            return
        
        # TODO: In a real implementation, we would open a browser or report viewer here
        # Since this is a proof of concept, just show a message with the report paths
        
        report_paths = []
        for directory in ["/media/usb/ServiceScan", "/hdd/ServiceScan", "/media/FTA/ServiceScan", "/tmp"]:
            if os.path.exists(directory):
                for report in reports:
                    if os.path.exists(os.path.join(directory, report)):
                        report_paths.append(os.path.join(directory, report))
        
        self.session.open(MessageBox, 
                         _("Found %d reports:\n%s") % (len(report_paths), "\n".join(report_paths)), 
                         MessageBox.TYPE_INFO, timeout=10)

# ReportViewer screen (simplified for proof of concept)
class ReportViewer(Screen):
    skin = """
    <screen position="center,center" size="1224,979" title="Enhanced Scan Report Viewer">
        <widget name="report_text" position="17,17" size="1190,867" font="Regular;30" />
        <eLabel position="17,901" size="1190,2" backgroundColor="#aaaaaa" />
        <widget source="key_red" render="Label" position="17,918" size="238,44" backgroundColor="red" foregroundColor="white" font="Regular;34" />
        <widget source="key_green" render="Label" position="272,918" size="238,44" backgroundColor="green" foregroundColor="white" font="Regular;34" />
    </screen>"""
    
    def __init__(self, session, report_path):
        Screen.__init__(self, session)
        self.setTitle(_("Enhanced Scan Report Viewer"))
        
        self["key_red"] = StaticText(_("Close"))
        self["key_green"] = StaticText(_("Refresh"))
        
        # Simple text widget to show report content
        self["report_text"] = Label(_("Loading report..."))
        
        self["actions"] = ActionMap(["SetupActions", "ColorActions"],
        {
            "cancel": self.close,
            "red": self.close,
            "green": self.loadReport,
        })
        
        self.report_path = report_path
        self.loadReport()
    
    def loadReport(self):
        try:
            with open(self.report_path, 'r') as f:
                # For a real implementation, we'd parse and format the HTML nicely
                # Here we just show the first few lines
                content = f.readlines()
                text = "".join(content[:30])
                
                if len(content) > 30:
                    text += "\n...\n(Report truncated - view full report on a browser)"
                
                self["report_text"].setText(text)
        except:
            self["report_text"].setText(_("Failed to load report"))

# Plugin initialization
def main(session, **kwargs):
    session.open(ScanTSDuckSetup)

def autostart(reason, **kwargs):
    if reason == 0:  # Enigma2 startup
        if config.plugins.scantsduck.autostart.value:
            # Make sure the log directory exists
            if not os.path.exists('/tmp/scantsduck'):
                try:
                    os.makedirs('/tmp/scantsduck')
                except Exception as e:
                    print(f"Error creating log directory: {e}")
                    pass

            # Add a delay before starting the service
            from time import sleep
            sleep(10)  # Give system more time to initialize
            
            try:
                # Start the scan monitor service with error handling
                log_out = open('/tmp/scantsduck/stdout.log', 'a')
                log_err = open('/tmp/scantsduck/stderr.log', 'a')
                
                # Use shell=True to work around permission issues
                subprocess.Popen('sh /usr/bin/scan_monitor.sh', 
                               shell=True,
                               stdout=log_out,
                               stderr=log_err)
            except Exception as e:
                print(f"Error starting scan_monitor.sh: {e}")
                # Try alternative method if first one fails
                try:
                    os.system('sh /usr/bin/scan_monitor.sh > /tmp/scantsduck/stdout.log 2> /tmp/scantsduck/stderr.log &')
                except:
                    pass

def Plugins(**kwargs):
    result = [
        PluginDescriptor(
            name=_("ServiceScan TSDuck Integration"),
            description=_("Enhanced scanning with TSDuck analysis"),
            where=[PluginDescriptor.WHERE_PLUGINMENU],
            fnc=main
        ),
        PluginDescriptor(
            where=PluginDescriptor.WHERE_AUTOSTART,
            fnc=autostart
        )
    ]
    
    if config.plugins.scantsduck.show_in_extensionsmenu.value:
        result.append(
            PluginDescriptor(
                name=_("ServiceScan TSDuck"),
                description=_("Enhanced scanning with TSDuck analysis"),
                where=[PluginDescriptor.WHERE_EXTENSIONSMENU],
                fnc=main
            )
        )
    
    return result
